﻿Imports System.IO

Public Class ID3

    Public Class Header

        Structure SVersion
            Dim Major As Byte
            Dim Minor As Byte
        End Structure

        Public Version As SVersion
        Public Flags As Byte
        Public Size As Int32

        ReadOnly Property Extended As Boolean
            Get
                Return (Flags And 32) = 32
            End Get
        End Property

    End Class

    Public Class ExtendedHeader
        Public Size As Int32
        Public FlagCount As Int32
        Public Flags As Byte

        Public RestrictionFlags As Int32
        Public CRC As Int32

        ReadOnly Property isUpdate() As Boolean
            Get
                Return (Flags And 64) = 64
            End Get
        End Property

        ReadOnly Property CRCPresent() As Boolean
            Get
                Return (Flags And 32) = 32
            End Get
        End Property

        ReadOnly Property Restrictions() As Boolean
            Get
                Return (Flags And 16) = 16
            End Get
        End Property
    End Class

    Public Class Frame
        Public Name As String
        Public Size As Int32
        Public Flags0 As Byte
        Public Flags1 As Byte
        Public Data As String
        Public Encoding As Byte

        Private header As Header

        ReadOnly Property CommonName()
            Get
                For i As Int16 = 0 To commonNames.Length - 1 Step 2
                    If Name = commonNames(i) Then
                        Return commonNames(i + 1)
                    End If
                Next

                Return "Unknown"
            End Get
        End Property

        Sub New(ByVal h As Header)
            header = h
        End Sub
    End Class

    Private Shared versions() = {2, 3, 4}
    Private Shared commonNames() = { _
        "PCNT", "Wiedergabeanzahl", _
        "TRCK", "Track", _
        "TRK", "Track", _
        "TENC", "Encoder", _
        "TEN", "Encoder", _
        "WXXX", "Link", _
        "TCOP", "Copyright", _
        "TCR", "Copyright", _
        "TOPE", "Artist", _
        "TOA", "Artist", _
        "TCOM", "Komponist", _
        "TCO", "Komponist", _
        "TCON", "Genre", _
        "COMM", "Kommentar", _
        "COM", "Kommentar", _
        "TYER", "Jahr", _
        "TYE", "Jahr", _
        "TALB", "Album", _
        "TAL", "Album", _
        "TPE1", "Artist", _
        "TP1", "Artist", _
        "TIT2", "Titel", _
        "TT2", "Titel", _
        "TLAN", "Sprache", _
        "TLA", "Sprache", _
        "TLEN", "Länge", _
        "TMT", "Typ", _
        "TMED", "Typ", _
        "TPUB", "Herausgeber", _
        "TSSE", "Software", _
        "TSS", "Software"}

    Public Sub New()
        Throw New Exception
    End Sub

    'http://id3.org/id3v2.4.0-structure
    Shared Function getID3V2Tag(ByVal filename As String) As List(Of Frame)
        Dim header As New Header, exHeader As ExtendedHeader
        Dim fs As New FileStream(filename, FileMode.Open, FileAccess.Read)
        Dim br = New BinaryReader(fs)
        Dim list As New List(Of Frame)


        If Not File.Exists(filename) Or Path.GetExtension(filename).ToLower <> ".mp3" Then
            Return list
        End If

        Try
            If br.ReadChars(3) <> "ID3" Then
                'No ID3 V2 Tags..
                Return list
            End If
        Catch
            Debug.Print("foo @ " & filename)
        End Try

        header.Version.Major = br.ReadByte()
        header.Version.Minor = br.ReadByte()
        header.Flags = br.ReadByte()
        header.Size = readSynch4(br)

        'Extendet Header(Bit6)
        If header.Extended Then
            exHeader = getExtendedHeader(br)
        End If

        Do
            Dim fSize As Int32 = If(header.Version.Major = 2, 3, 4)
            Dim Frame = New Frame(header)

            Frame.Name = ByteArrayToString(br.ReadBytes(fSize))

            'Padding gefunden..
            If Frame.Name = New String(Chr(0), fSize) Then
                Exit Do
            ElseIf Text.RegularExpressions.Regex.Match(Frame.Name, "[^A-Z0-9]").Success Then
                'Fehler abfangen...
                Exit Do
            End If

            Frame.Size = If(header.Version.Major = 2, readSynch3(br), readSynch4(br))
            Frame.Flags0 = br.ReadByte
            Frame.Flags1 = br.ReadByte

            Try
                If Frame.Size = 0 Then
                    Frame.Data = ""
                    Frame.Encoding = 0
                Else
                    Frame.Encoding = br.ReadByte
                    If Frame.Encoding < 4 Then
                        Frame.Data = ByteArrayToString(br.ReadBytes(Frame.Size - 1))
                    Else
                        Frame.Data = Chr(Frame.Encoding) + ByteArrayToString(br.ReadBytes(Frame.Size - 1))
                    End If
                End If


            Catch e As Exception
                MessageBox.Show(e.Message)
            End Try

            list.Add(Frame)

        Loop While (br.BaseStream.Position <= Convert.ToInt64(header.Size))

        fs.Close()

        Return list
    End Function

    Private Shared Function ByteArrayToString(ByVal Data() As Byte) As String
        Dim str = New System.Text.StringBuilder(Data.Length)
        For i As Integer = 0 To Data.Length - 1
            str.Append(Chr(Data(i)))
        Next

        Return str.ToString
    End Function


    Private Shared Function getExtendedHeader(ByVal br As BinaryReader) As ExtendedHeader
        Dim eh = New ExtendedHeader

        eh.Size = readSynch4(br)
        eh.FlagCount = Convert.ToInt32(br.ReadByte())

        eh.Flags = br.ReadByte

        If eh.isUpdate Then
            br.ReadByte()
        End If

        If eh.CRCPresent Then
            Dim crcSize = 0
            If Convert.ToInt32(br.ReadByte) <> 5 Then
                'Defekter CRC?
                Throw New Exception
            End If

            eh.Size = readSynch5(br)

        End If

        If eh.Restrictions Then
            br.ReadByte()
            eh.RestrictionFlags = br.ReadByte
        End If

        Return eh
    End Function

    Private Shared Function readSynch3(ByVal br As BinaryReader) As Int32
        Return br.ReadByte() * &H4000 + br.ReadByte * &H80 + br.ReadByte()
    End Function

    Private Shared Function readSynch4(ByVal br As BinaryReader) As Int32
        Return br.ReadByte() * &H200000 + br.ReadByte() * &H4000 + br.ReadByte * &H80 + br.ReadByte()
    End Function

    Private Shared Function readSynch5(ByVal br As BinaryReader) As Int32
        Return br.ReadByte() * &H10000000 + br.ReadByte() * &H200000 + br.ReadByte() * &H4000 + br.ReadByte * &H80 + br.ReadByte()
    End Function


End Class

