﻿Imports System.Xml
Imports System.Xml.XPath
Imports System.IO

Public Class Playlist

    Private list As List(Of Media.mediaElement)
    Private nowPlaying As Media.mediaElement
    Private firstItem As Media.mediaElement
    Private lastItem As Media.mediaElement

    Private bShuffle As Boolean = False

    Sub New()
        list = New List(Of Media.mediaElement)
    End Sub


    Sub Clear()
        list.Clear()
        nowPlaying = Nothing
        firstItem = Nothing
        lastItem = Nothing
    End Sub


    ReadOnly Property Length As Integer
        Get
            Return list.Count
        End Get
    End Property

    Public Sub removeDuplicates()
        Dim i, j As Integer

        While (i < list.Count)
            j = i + 1
            While (j < list.Count)
                If list(i).src = list(j).src Then
                    list.RemoveAt(j)
                Else
                    j += 1
                End If
            End While
            i += 1
        End While
    End Sub


    Shared Function fromFile(ByVal filename As String) As Playlist
        Dim pl As New Playlist

        If filename.EndsWith(".m3u", StringComparison.OrdinalIgnoreCase) Then
            Return fromFileM3U(filename, pl)
        ElseIf filename.EndsWith(".wpl", StringComparison.OrdinalIgnoreCase) Then
            Return fromFileWpl(filename, pl)
        End If


        Return Nothing
    End Function

    Private Shared Function fromFileWpl(ByVal filename As String, ByVal pl As Playlist)
        Dim doc As XPathDocument
        Dim nav As XPathNavigator
        Dim iter As XPathNodeIterator
        Dim media As String
        Dim relPath = Path.GetDirectoryName(filename)

        If Not File.Exists(filename) Then
            Return Nothing
        End If

        Try
            doc = New XPathDocument(filename)
            nav = doc.CreateNavigator()

            iter = nav.Select("/smil/body/seq/*")

            While (iter.MoveNext)
                If iter.Current.Name = "media" Then
                    media = iter.Current.GetAttribute("src", "")

                    Dim s As DirectoryInfo

                    If media.Substring(1, 1) = ":" Then
                        s = New DirectoryInfo(media)
                    Else
                        s = New DirectoryInfo(relPath + Path.DirectorySeparatorChar + media)
                    End If

                    Dim id = ID3.getID3V2Tag(s.FullName)
                    pl.addFile(Controller.instance().Library.addMediaElement(s.FullName))

                End If
            End While

        Catch e As Exception
            MessageBox.Show(e.Message)
            Return pl
        End Try

        Return pl
    End Function

    Private Shared Function fromFileM3U(ByVal filename As String, ByVal pl As Playlist)
        If Not File.Exists(filename) Then
            Return New Playlist
        End If

        Dim relPath = Path.GetDirectoryName(filename)
        Dim fs = Encoding.detectFile(New FileStream(filename, FileMode.Open, FileAccess.Read))

        If Not fs.ReadLine = "#EXTM3U" Then
            Return pl
        End If

        While (Not fs.EndOfStream)
            Dim line As String = fs.ReadLine

            If line.Length = 0 Then
                Continue While
            End If

            If line.StartsWith("#EXTINF:") Then
                Dim file = fs.ReadLine
                Dim s As DirectoryInfo
                Dim ex = line.Split(",")
                Dim title As String

                If ex.Length = 2 Then
                    title = ex(1)
                Else
                    title = Path.GetFileName(file)
                End If

                If file.Substring(1, 1) <> ":" Then
                    file = relPath + Path.DirectorySeparatorChar + file
                End If

                If Not System.IO.File.Exists(file) Then
                    Continue While
                End If

                s = New DirectoryInfo(file)
                pl.addFile(Controller.instance().Library.addMediaElement(s.FullName))
            End If
        End While

        fs.Close()

        Return pl
    End Function

    Sub savePlaylistM3U(ByVal filename As String)
        Dim fs = New StreamWriter(filename, False)
        fs.WriteLine("#EXTM3U")

        For Each item As Media.mediaElement In list
            fs.WriteLine("#EXTINF:0," & Path.GetFileName(item.Filename))
            fs.WriteLine(item.Filename & vbCrLf)
        Next

        fs.Close()
    End Sub


    Function addFile(ByVal media As Media.mediaElement) As Media.mediaElement
        If media Is Nothing Then
            Return Nothing
        End If

        Me.list.Add(media)

        If firstItem Is Nothing Then
            firstItem = media
        End If
        lastItem = media

        Return media
    End Function

    Function getNext(Optional ByVal media As Media.mediaElement = Nothing) As Media.mediaElement
        If Me.Length = 0 Then
            Return Nothing
        End If

        Dim idx = list.IndexOf(Me.nowPlaying)
        Dim mnext As Media.mediaElement
        If media Is Nothing Then
            If bShuffle Then
                mnext = list((New Random()).Next(0, list.Count - 1))
            Else
                If idx < 0 Then
                    mnext = firstItem
                ElseIf idx + 1 < Length Then
                    mnext = list(idx + 1)
                Else
                    mnext = firstItem
                End If
            End If
        Else
            mnext = media
        End If

        mnext.prevMedia = nowPlaying
        If nowPlaying IsNot Nothing Then
            nowPlaying.nextMedia = mnext
        End If

        nowPlaying = mnext

        Return mnext
    End Function

    Function getPrev() As Media.mediaElement
        nowPlaying = nowPlaying.prevMedia
        Return nowPlaying
    End Function

    Function isPlaying() As Boolean
        Return nowPlaying IsNot Nothing
    End Function

    Property Shuffle() As Boolean
        Get
            Return bShuffle
        End Get
        Set(ByVal value As Boolean)
            bShuffle = value
        End Set
    End Property

    ReadOnly Property current As Media.mediaElement
        Get
            Return nowPlaying
        End Get
    End Property

    ReadOnly Property items As List(Of Media.mediaElement)
        Get
            Return list
        End Get
    End Property


End Class
